/*
 * Written by Eeri Kask ...--2012
 *
 * thread-safe multi-platform random generator
 * using 48-bit arithmetic
 *
 */

#ifndef __ED_RAND
#define __ED_RAND

#define RANDOM48_A0             (0xe66d) /* least significant part */
#define RANDOM48_A1             (0xdeec)
#define RANDOM48_A2             (0x0005) /* most significant part  */
#define RANDOM48_C              (0x000b)

class ed_rand {

    typedef unsigned short int      UINT16;
    typedef unsigned int            UINT32;
    typedef unsigned long long int  UINT64;

    unsigned short int state[3];

    inline void generate() {
        register  UINT32 carry;
        register  UINT16 low16, mid16;

        carry  =  (UINT32)(RANDOM48_C);

        carry +=  (UINT32)(RANDOM48_A0) * state[0];
        low16  =  (UINT16)(carry);

        carry >>= (sizeof(low16) << 3);

        carry +=  (UINT32)(RANDOM48_A0) * state[1]  +  (UINT32)(RANDOM48_A1) * state[0];
        mid16  =  (UINT16)(carry);

        carry >>= (sizeof(mid16) << 3);

        carry  += (UINT32)(RANDOM48_A0) * state[2]  +  (UINT32)(RANDOM48_A1) * state[1]  +  (UINT32)(RANDOM48_A2) * state[0];

        state[0] = low16;
        state[1] = mid16;
        state[2] = (UINT16)(carry);
    };

public:
    ed_rand(unsigned int seed) {
        state[0]=(unsigned short int)(seed&0xFFFF);
        state[1]=(unsigned short int)((seed>>8)&0xFFFF);
        state[2]=(unsigned short int)((seed>>16)&0xFFFF);
    };

    ~ed_rand() {};

    inline unsigned short int rand() {
        generate();
        return state[1];
    };

    inline double drand() {
        union {
            UINT16 s[4];
            UINT64 i;
            double d;
        } result;

        generate();

        result.i = state[0];
        /* on lo-endian this is s[0], on hi-endian this is s[3];
         on hi-endian the following s[1], s[2] should be swapped if it matters */
        result.s[1] = state[1];
        result.s[2] = state[2];
        return (double)(result.i) / (double)((UINT64)0x1000000000000);
    };
};

#endif
