/*
 * Written by Eeri Kask ...--2012
 *
 * a minimalistic wrapper for threading management
 * a mapping to "POSIX threads" under MAC/Linux
 * and Win32-threads under Windows
 */

#ifndef __ED_THREAD
#define __ED_THREAD

#if defined (_WIN32) || defined (_WIN64)
#include <windows.h>
#else
#include <pthread.h>
#endif

class ed_thread {
public:
    // Constructor: Create a 'suspended' thread ready to execute
    ed_thread();

    // Destructor: Destroy thread
    ~ed_thread();

    // thread management
#if defined (_WIN32) || defined (_WIN64)
    HANDLE          thrhnd;
    DWORD           id;
    HANDLE          msg[3];
#else
    pthread_t       id;
    pthread_cond_t  cond_run;
    pthread_mutex_t mutx_run;
    // thread state:
    // 0 = 'waiting/suspended';  1 = 'running/activated';  2 = 'terminating/destructed'
    unsigned        stat_run;
#endif

    // it should be implemented in user classes
    virtual void run() = 0;

    // Activate a 'suspended' thread to execute 'run()'
    void start();

    // Wait for a 'running' thread to return from 'run()' into 'suspended' state
    void wait();

    // Return number of available processor execution units
    static int num_threads();
};

#endif
