/* Written by Dmitrij Schlesinger (2012),
 * http://wwwpub.zih.tu-dresden.de/~ds24/rtsegm/rtsegm.html
*/

#ifndef __RTSEGM_GAUSS
#define __RTSEGM_GAUSS

#include "ed_thread.h"

// This class implements Metropolis Sampling for Gaussian numbers
class Rtsegm_gauss : public ed_thread {
public:
    Rtsegm_gauss(int height, int width, int ng, double step);
    virtual ~Rtsegm_gauss();

    // thread management
    bool stop_me;
    void run();
    void reset();
    int it_count;

    // auxiliary function to calculate coefficients for scalar products
    // used in "::run()"
    void do_coeff();

    // modell data
    int width, height;
    // the input image (captured by the main thread)
    unsigned char *img;
    // the number of Gaussians
    int ng;
    // mean values
    double *m_r, *m_g, *m_b;
    // dispersion
    double sigma;
    // logarithms of Gaussian weights
    double *weights;

    // coefficients for scalar products used in "::run()"
    double *kr, *kg, *kb, *t;

    // field of Gaussian numbers
    int *glab;
    // the current segmentation (sampled by "Rtsegm_segm")
    int *lab;

    // learning step
    double step;
    // "noise level" (see the implementation for details)
    double gnoise;
    // accumulators for statistics
    double *acc_r, *acc_g, *acc_b, *acc_weights, acc_sigma;

    // lookup table
    unsigned short int *table;
};

#endif
