/* Written by Dmitrij Schlesinger (2012),
 * http://wwwpub.zih.tu-dresden.de/~ds24/rtsegm/rtsegm.html
*/

#include <stdlib.h>
#include <time.h>
#include <iostream>
using namespace std;
#include <math.h>
#include "ed_rand.h"
#include "rtsegm_segm.h"

// constructor -- remember parameters, allocate arrays, initialize randomly
Rtsegm_segm::Rtsegm_segm(int height, int width, double potts, double step)
    : ed_thread() {
    this->height=height;
    this->width=width;
    this->potts=potts;
    this->step=step;
    potts2=potts*2.;

    lab=new int[height*width];
    marg=new double[height*width];
    ed_rand *generator=new ed_rand((unsigned int)time(0)+12345);
    for( int i=0; i<height*width; i++ ) {
        lab[i]=generator->rand()&1;
        marg[i]=0.5;
    }
    delete generator;

    stop_me=false;
    it_count=0;
    qsh=0;

    // lookup table
    table=new unsigned short[0x10000]; // 128k
    for( int i=0; i<0x10000; i++ ) table[i]=(unsigned short)(0xFFFF/(exp(-i/1000.)+1.)+0.5);
}

// destructor -- clean all
Rtsegm_segm::~Rtsegm_segm() {
    delete lab;
    delete marg;
    delete table;
}

// the entry point for separate thread -- Gibbs Sampling for labellings
void Rtsegm_segm::run() {
    // allocate and initialize the own random generator (to be thread-safe)
    ed_rand *generator=new ed_rand((unsigned int)time(0)+12345);

    stop_me=false;

    // the main loop
    while( !stop_me ) {
        // scan over the image
        int hw=height*width;
        for( int i=0; i<hw; i++ ) {
            // observation
            int gi=glab[i]<<1;
            double e1=weights[gi+1]-weights[gi];

            // shape prior (if any)
            if( qsh ) e1+=qsh[i];

            // Potts prior
            int x=i%width;
            if( x ) e1+=potts2*lab[i-1]-potts;
            if( x<width-1 ) e1+=potts2*lab[i+1]-potts;
            int y=i/width;
            if( y ) e1+=potts2*lab[i-width]-potts;
            if( y<height-1 ) e1+=potts2*lab[i+width]-potts;

            // sample
            if( e1<0 ) {
                e1=-e1*1000.+0.5;
                unsigned int index;
                if( e1>=(double)0x10000 ) index=0xFFFF;
                else index=(int)e1;
                if( generator->rand()>table[index] ) lab[i]=1;
                else lab[i]=0;
            } else {
                e1=e1*1000.+0.5;
                unsigned int index;
                if( e1>=(double)0x10000 ) index=0xFFFF;
                else index=(int)e1;
                if( generator->rand()>table[index] ) lab[i]=0;
                else lab[i]=1;
            }

            // accumulate label marginals
            marg[i]=(1.-step)*marg[i]+step*lab[i];
        }
        it_count++;
    }

    stop_me=false;
    delete generator;
}

// re-initialize the segmentation randomly
void Rtsegm_segm::reset() {
    ed_rand *generator=new ed_rand((unsigned int)time(0)+12345);
    for( int i=0; i<height*width; i++ ) {
        lab[i]=generator->rand()&1;
        marg[i]=0.5;
    }
    delete generator;

    stop_me=false;
}
