/* Written by Dmitrij Schlesinger (2012-2013),
 * http://wwwpub.zih.tu-dresden.de/~ds24/rtsegm/rtsegm.html
*/

#ifndef __RTSEGM_SHAPE
#define __RTSEGM_SHAPE

#include "ed_thread.h"

// use look-up table for exponent
// (much faster but may lose precision)
#define USE_LOOKUP

// This class implements the shape estimation
class Rtsegm_shape : public ed_thread {
public:
    Rtsegm_shape(int height, int width,
				 double step_m, double step_g, double step_d, double beta);
    virtual ~Rtsegm_shape();

    // thread management
    bool stop_me;
    void run();
    void reset();
    int it_count;

    // modell data
    int width, height;

	// current segmentation (sampled by "Rtsegm_segm")
    int *lab;
    
	// the additional unary potentials for the foregroung
    // (used by "Rtsegm_segm")
    double *qsh;
    
	// the range of the shape function
	// (for the visualization by the main thread)
	double min_s, max_s;

	// learning steps
    double step_m, step_g, step_d;
	
	// the constant for the approximation of the prior marginals
	// see Eq. 7 in the EMMCVPR paper
	double beta;

    // temporary fields
    double *tmp_g;
	double *tmp_d;
	
#ifdef USE_LOOKUP	
	// look-up table
	double *lt;
#endif

    // convolutions
    void filter_g(double *src, double *dest); // for gradients
    void filter_d(double *src, double *dest); // for Laplacians
};

#endif
